@@ -11,6 +11,8 @@ module Network.HTTP2.Client.Internal (
     runIO,
 ) where
 
+import Network.HTTP.Semantics.Client
+import Network.HTTP.Semantics.Client.Internal
+
 import Network.HTTP2.Client.Run
-import Network.HTTP2.Client.Types
 import Network.HTTP2.H2
