@@ -1,10 +1,10 @@
-{-# LANGUAGE DeriveDataTypeable #-}
 {-# LANGUAGE OverloadedStrings #-}
 {-# LANGUAGE PatternSynonyms #-}
 
 module Network.HTTP2.Frame.Types where
 
 import Data.Ix
+import Network.Control (WindowSize)
 import Text.Read
 import qualified Text.Read.Lex as L
 
@@ -32,6 +32,7 @@ toErrorCode = ErrorCode
 
 -- | The type for error code. See <https://www.rfc-editor.org/rfc/rfc9113#ErrorCodes>.
 
+{- FOURMOLU_DISABLE -}
 pattern NoError            :: ErrorCode
 pattern NoError             = ErrorCode 0x0
 
@@ -73,23 +74,26 @@ pattern InadequateSecurity  = ErrorCode 0xc
 
 pattern HTTP11Required     :: ErrorCode
 pattern HTTP11Required      = ErrorCode 0xd
+{- FOURMOLU_ENABLE -}
 
+{- FOURMOLU_DISABLE -}
 instance Show ErrorCode where
-    show (ErrorCode 0x0) = "NoError"
-    show (ErrorCode 0x1) = "ProtocolError"
-    show (ErrorCode 0x2) = "InternalError"
-    show (ErrorCode 0x3) = "FlowControlError"
-    show (ErrorCode 0x4) = "SettingsTimeout"
-    show (ErrorCode 0x5) = "StreamClosed"
-    show (ErrorCode 0x6) = "FrameSizeError"
-    show (ErrorCode 0x7) = "RefusedStream"
-    show (ErrorCode 0x8) = "Cancel"
-    show (ErrorCode 0x9) = "CompressionError"
-    show (ErrorCode 0xa) = "ConnectError"
-    show (ErrorCode 0xb) = "EnhanceYourCalm"
-    show (ErrorCode 0xc) = "InadequateSecurity"
-    show (ErrorCode 0xd) = "HTTP11Required"
-    show (ErrorCode   x) = "ErrorCode " ++ show x
+    show NoError            = "NoError"
+    show ProtocolError      = "ProtocolError"
+    show InternalError      = "InternalError"
+    show FlowControlError   = "FlowControlError"
+    show SettingsTimeout    = "SettingsTimeout"
+    show StreamClosed       = "StreamClosed"
+    show FrameSizeError     = "FrameSizeError"
+    show RefusedStream      = "RefusedStream"
+    show Cancel             = "Cancel"
+    show CompressionError   = "CompressionError"
+    show ConnectError       = "ConnectError"
+    show EnhanceYourCalm    = "EnhanceYourCalm"
+    show InadequateSecurity = "InadequateSecurity"
+    show HTTP11Required     = "HTTP11Required"
+    show (ErrorCode x)      = "ErrorCode " ++ show x
+{- FOURMOLU_ENABLE -}
 
 ----------------------------------------------------------------
 
@@ -108,8 +112,9 @@ minSettingsKey = SettingsKey 1
 maxSettingsKey :: SettingsKey
 maxSettingsKey = SettingsKey 6
 
-pattern SettingsHeaderTableSize      :: SettingsKey
-pattern SettingsHeaderTableSize       = SettingsKey 1
+{- FOURMOLU_DISABLE -}
+pattern SettingsTokenHeaderTableSize :: SettingsKey
+pattern SettingsTokenHeaderTableSize  = SettingsKey 1
 
 pattern SettingsEnablePush           :: SettingsKey
 pattern SettingsEnablePush            = SettingsKey 2
@@ -123,17 +128,20 @@ pattern SettingsInitialWindowSize     = SettingsKey 4
 pattern SettingsMaxFrameSize         :: SettingsKey
 pattern SettingsMaxFrameSize          = SettingsKey 5 -- this means payload size
 
-pattern SettingsMaxHeaderBlockSize   :: SettingsKey
-pattern SettingsMaxHeaderBlockSize    = SettingsKey 6
+pattern SettingsMaxHeaderListSize    :: SettingsKey
+pattern SettingsMaxHeaderListSize     = SettingsKey 6
+{- FOURMOLU_ENABLE -}
 
+{- FOURMOLU_DISABLE -}
 instance Show SettingsKey where
-    show SettingsHeaderTableSize      = "SettingsHeaderTableSize"
+    show SettingsTokenHeaderTableSize = "SettingsTokenHeaderTableSize"
     show SettingsEnablePush           = "SettingsEnablePush"
     show SettingsMaxConcurrentStreams = "SettingsMaxConcurrentStreams"
     show SettingsInitialWindowSize    = "SettingsInitialWindowSize"
     show SettingsMaxFrameSize         = "SettingsMaxFrameSize"
-    show SettingsMaxHeaderBlockSize   = "SettingsMaxHeaderBlockSize"
+    show SettingsMaxHeaderListSize    = "SettingsMaxHeaderListSize"
     show (SettingsKey x)              = "SettingsKey " ++ show x
+{- FOURMOLU_ENABLE -}
 
 instance Read SettingsKey where
     readListPrec = readListPrecDefault
@@ -141,67 +149,22 @@ instance Read SettingsKey where
         Ident idnt <- lexP
         readSK idnt
       where
-        readSK "SettingsHeaderTableSize"      = return SettingsHeaderTableSize
-        readSK "SettingsEnablePush"           = return SettingsEnablePush
+        readSK "SettingsTokenHeaderTableSize" = return SettingsTokenHeaderTableSize
+        readSK "SettingsEnablePush" = return SettingsEnablePush
         readSK "SettingsMaxConcurrentStreams" = return SettingsMaxConcurrentStreams
-        readSK "SettingsInitialWindowSize"    = return SettingsInitialWindowSize
-        readSK "SettingsMaxFrameSize"         = return SettingsMaxFrameSize
-        readSK "SettingsMaxHeaderBlockSize"   = return SettingsMaxHeaderBlockSize
-        readSK "SettingsKey"         = do
-              Number ftyp <- lexP
-              return $ SettingsKey $ fromIntegral $ fromJust $ L.numberToInteger ftyp
-        readSK _                   = error "Read for SettingsKey"
+        readSK "SettingsInitialWindowSize" = return SettingsInitialWindowSize
+        readSK "SettingsMaxFrameSize" = return SettingsMaxFrameSize
+        readSK "SettingsMaxHeaderListSize" = return SettingsMaxHeaderListSize
+        readSK "SettingsKey" = do
+            Number ftyp <- lexP
+            return $ SettingsKey $ fromIntegral $ fromJust $ L.numberToInteger ftyp
+        readSK _ = error "Read for SettingsKey"
 
 -- | The type for raw SETTINGS value.
 type SettingsValue = Int -- Word32
 
 -- | Association list of SETTINGS.
-type SettingsList = [(SettingsKey,SettingsValue)]
-
-----------------------------------------------------------------
-
--- | Cooked version of settings. This is suitable to be stored in a HTTP/2 context.
-data Settings = Settings {
-    headerTableSize :: Int
-  , enablePush :: Bool
-  , maxConcurrentStreams :: Maybe Int
-  , initialWindowSize :: WindowSize
-  , maxFrameSize :: Int
-  , maxHeaderListSize :: Maybe Int
-  } deriving (Show)
-
--- | The default settings.
---
--- >>> defaultSettings
--- Settings {headerTableSize = 4096, enablePush = True, maxConcurrentStreams = Nothing, initialWindowSize = 65535, maxFrameSize = 16384, maxHeaderListSize = Nothing}
-defaultSettings :: Settings
-defaultSettings = Settings {
-    headerTableSize = 4096 -- defaultDynamicTableSize
-  , enablePush = True
-  , maxConcurrentStreams = Nothing
-  , initialWindowSize = defaultWindowSize
-  , maxFrameSize = defaultPayloadLength
-  , maxHeaderListSize = Nothing
-  }
-
--- | Updating settings.
---
--- >>> updateSettings defaultSettings [(SettingsEnablePush,0),(SettingsMaxHeaderBlockSize,200)]
--- Settings {headerTableSize = 4096, enablePush = False, maxConcurrentStreams = Nothing, initialWindowSize = 65535, maxFrameSize = 16384, maxHeaderListSize = Just 200}
-updateSettings :: Settings -> SettingsList -> Settings
-updateSettings settings kvs = foldl' update settings kvs
-  where
-    update def (SettingsHeaderTableSize,x)      = def { headerTableSize = x }
-    -- fixme: x should be 0 or 1
-    update def (SettingsEnablePush,x)           = def { enablePush = x > 0 }
-    update def (SettingsMaxConcurrentStreams,x) = def { maxConcurrentStreams = Just x }
-    update def (SettingsInitialWindowSize,x)    = def { initialWindowSize = x }
-    update def (SettingsMaxFrameSize,x)         = def { maxFrameSize = x }
-    update def (SettingsMaxHeaderBlockSize,x)   = def { maxHeaderListSize = Just x }
-    update def _                                = def
-
--- | The type for window size.
-type WindowSize = Int
+type SettingsList = [(SettingsKey, SettingsValue)]
 
 -- | The default initial window size.
 --
@@ -226,8 +189,7 @@ maxWindowSize = 2147483647
 -- >>> isWindowOverflow (maxWindowSize + 1)
 -- True
 isWindowOverflow :: WindowSize -> Bool
-isWindowOverflow w = testBit w 31
-
+isWindowOverflow w = w > maxWindowSize
 
 -- | Default concurrency.
 --
@@ -243,11 +205,12 @@ recommendedConcurrency = 100
 type Weight = Int
 
 -- | Type for stream priority. Deprecated in RFC 9113 but provided for 'FrameHeaders'.
-data Priority = Priority {
-    exclusive :: Bool
-  , streamDependency :: StreamId
-  , weight :: Weight
-  } deriving (Show, Read, Eq)
+data Priority = Priority
+    { exclusive :: Bool
+    , streamDependency :: StreamId
+    , weight :: Weight
+    }
+    deriving (Show, Read, Eq)
 
 ----------------------------------------------------------------
 
@@ -272,6 +235,7 @@ minFrameType = FrameType 0
 maxFrameType :: FrameType
 maxFrameType = FrameType 9
 
+{- FOURMOLU_DISABLE -}
 pattern FrameData         :: FrameType
 pattern FrameData          = FrameType 0
 
@@ -301,20 +265,24 @@ pattern FrameWindowUpdate  = FrameType 8
 
 pattern FrameContinuation :: FrameType
 pattern FrameContinuation  = FrameType 9
+{- FOURMOLU_ENABLE -}
 
+{- FOURMOLU_DISABLE -}
 instance Show FrameType where
-    show (FrameType 0) = "FrameData"
-    show (FrameType 1) = "FrameHeaders"
-    show (FrameType 2) = "FramePriority"
-    show (FrameType 3) = "FrameRSTStream"
-    show (FrameType 4) = "FrameSettings"
-    show (FrameType 5) = "FramePushPromise"
-    show (FrameType 6) = "FramePing"
-    show (FrameType 7) = "FrameGoAway"
-    show (FrameType 8) = "FrameWindowUpdate"
-    show (FrameType 9) = "FrameContinuation"
-    show (FrameType x) = "FrameType " ++ show x
+    show FrameData         = "FrameData"
+    show FrameHeaders      = "FrameHeaders"
+    show FramePriority     = "FramePriority"
+    show FrameRSTStream    = "FrameRSTStream"
+    show FrameSettings     = "FrameSettings"
+    show FramePushPromise  = "FramePushPromise"
+    show FramePing         = "FramePing"
+    show FrameGoAway       = "FrameGoAway"
+    show FrameWindowUpdate = "FrameWindowUpdate"
+    show FrameContinuation = "FrameContinuation"
+    show (FrameType x)     = "FrameType " ++ show x
+{- FOURMOLU_ENABLE -}
 
+{- FOURMOLU_DISABLE -}
 instance Read FrameType where
     readListPrec = readListPrecDefault
     readPrec = do
@@ -335,6 +303,7 @@ instance Read FrameType where
               Number ftyp <- lexP
               return $ FrameType $ fromIntegral $ fromJust $ L.numberToInteger ftyp
         readFT _                   = error "Read for FrameType"
+{- FOURMOLU_ENABLE -}
 
 ----------------------------------------------------------------
 
@@ -343,14 +312,14 @@ instance Read FrameType where
 -- >>> maxPayloadLength
 -- 16777215
 maxPayloadLength :: Int
-maxPayloadLength = 2^(24::Int) - 1
+maxPayloadLength = 2 ^ (24 :: Int) - 1
 
 -- | The default payload length of HTTP/2 payload.
 --
 -- >>> defaultPayloadLength
 -- 16384
 defaultPayloadLength :: Int
-defaultPayloadLength = 2^(14::Int)
+defaultPayloadLength = 2 ^ (14 :: Int)
 
 ----------------------------------------------------------------
 -- Flags
@@ -491,31 +460,33 @@ type Padding = ByteString
 
 -- | The data type for HTTP/2 frames.
 data Frame = Frame
-    { frameHeader  :: FrameHeader
+    { frameHeader :: FrameHeader
     , framePayload :: FramePayload
-    } deriving (Show, Read, Eq)
+    }
+    deriving (Show, Read, Eq)
 
 -- | The data type for HTTP/2 frame headers.
 data FrameHeader = FrameHeader
     { payloadLength :: Int
-    , flags         :: FrameFlags
-    , streamId      :: StreamId
-    } deriving (Show, Read, Eq)
+    , flags :: FrameFlags
+    , streamId :: StreamId
+    }
+    deriving (Show, Read, Eq)
 
 -- | The data type for HTTP/2 frame payloads.
-data FramePayload =
-    DataFrame ByteString
-  | HeadersFrame (Maybe Priority) HeaderBlockFragment
-  | PriorityFrame Priority
-  | RSTStreamFrame ErrorCode
-  | SettingsFrame SettingsList
-  | PushPromiseFrame StreamId HeaderBlockFragment
-  | PingFrame ByteString
-  | GoAwayFrame {- the last -}StreamId ErrorCode ByteString
-  | WindowUpdateFrame WindowSize
-  | ContinuationFrame HeaderBlockFragment
-  | UnknownFrame FrameType ByteString
-  deriving (Show, Read, Eq)
+data FramePayload
+    = DataFrame ByteString
+    | HeadersFrame (Maybe Priority) HeaderBlockFragment
+    | PriorityFrame Priority
+    | RSTStreamFrame ErrorCode
+    | SettingsFrame SettingsList
+    | PushPromiseFrame StreamId HeaderBlockFragment
+    | PingFrame ByteString
+    | GoAwayFrame {- the last -} StreamId ErrorCode ByteString
+    | WindowUpdateFrame WindowSize
+    | ContinuationFrame HeaderBlockFragment
+    | UnknownFrame FrameType ByteString
+    deriving (Show, Read, Eq)
 
 ----------------------------------------------------------------
 
@@ -523,6 +494,7 @@ data FramePayload =
 --
 -- >>> framePayloadToFrameType (DataFrame "body")
 -- FrameData
+{- FOURMOLU_DISABLE -}
 framePayloadToFrameType :: FramePayload -> FrameType
 framePayloadToFrameType DataFrame{}          = FrameData
 framePayloadToFrameType HeadersFrame{}       = FrameHeaders
@@ -535,6 +507,7 @@ framePayloadToFrameType GoAwayFrame{}        = FrameGoAway
 framePayloadToFrameType WindowUpdateFrame{}  = FrameWindowUpdate
 framePayloadToFrameType ContinuationFrame{}  = FrameContinuation
 framePayloadToFrameType (UnknownFrame ft _)  = ft
+{- FOURMOLU_ENABLE -}
 
 ----------------------------------------------------------------
 
@@ -544,6 +517,7 @@ framePayloadToFrameType (UnknownFrame ft _)  = ft
 -- True
 -- >>> isPaddingDefined $ PingFrame ""
 -- False
+{- FOURMOLU_DISABLE -}
 isPaddingDefined :: FramePayload -> Bool
 isPaddingDefined DataFrame{}         = True
 isPaddingDefined HeadersFrame{}      = True
@@ -556,13 +530,16 @@ isPaddingDefined GoAwayFrame{}       = False
 isPaddingDefined WindowUpdateFrame{} = False
 isPaddingDefined ContinuationFrame{} = False
 isPaddingDefined UnknownFrame{}      = False
+{- FOURMOLU_ENABLE -}
 
 ----------------------------------------------------------------
 -- Deprecated
 
+{- FOURMOLU_DISABLE -}
 type ErrorCodeId   = ErrorCode
 type SettingsKeyId = SettingsKey
 type FrameTypeId   = FrameType
-{- DEPRECATED ErrorCodeId   "Use ErrorCode instead" -}
-{- DEPRECATED SettingsKeyId "Use SettingsKey instead" -}
-{- DEPRECATED FrameTypeId   "Use FrameType instead" -}
+{- FOURMOLU_ENABLE -}
+{-# DEPRECATED ErrorCodeId "Use ErrorCode instead" #-}
+{-# DEPRECATED SettingsKeyId "Use SettingsKey instead" #-}
+{-# DEPRECATED FrameTypeId "Use FrameType instead" #-}
