@@ -2,17 +2,14 @@ module Network.HTTP2.H2 (
     module Network.HTTP2.H2.Config,
     module Network.HTTP2.H2.Context,
     module Network.HTTP2.H2.EncodeFrame,
-    module Network.HTTP2.H2.File,
     module Network.HTTP2.H2.HPACK,
-    module Network.HTTP2.H2.Manager,
     module Network.HTTP2.H2.Queue,
-    module Network.HTTP2.H2.ReadN,
     module Network.HTTP2.H2.Receiver,
     module Network.HTTP2.H2.Sender,
     module Network.HTTP2.H2.Settings,
-    module Network.HTTP2.H2.Status,
     module Network.HTTP2.H2.Stream,
     module Network.HTTP2.H2.StreamTable,
+    module Network.HTTP2.H2.Sync,
     module Network.HTTP2.H2.Types,
     module Network.HTTP2.H2.Window,
 ) where
@@ -20,16 +17,13 @@ module Network.HTTP2.H2 (
 import Network.HTTP2.H2.Config
 import Network.HTTP2.H2.Context
 import Network.HTTP2.H2.EncodeFrame
-import Network.HTTP2.H2.File
 import Network.HTTP2.H2.HPACK
-import Network.HTTP2.H2.Manager
 import Network.HTTP2.H2.Queue
-import Network.HTTP2.H2.ReadN
 import Network.HTTP2.H2.Receiver
 import Network.HTTP2.H2.Sender
 import Network.HTTP2.H2.Settings
-import Network.HTTP2.H2.Status
 import Network.HTTP2.H2.Stream
 import Network.HTTP2.H2.StreamTable
+import Network.HTTP2.H2.Sync
 import Network.HTTP2.H2.Types
 import Network.HTTP2.H2.Window
