@@ -2,23 +2,26 @@
 {-# LANGUAGE OverloadedStrings #-}
 {-# LANGUAGE PatternGuards #-}
 {-# LANGUAGE RecordWildCards #-}
+{-# LANGUAGE ScopedTypeVariables #-}
 
 module Network.HTTP2.H2.Receiver (
     frameReceiver,
+    closureClient,
+    closureServer,
 ) where
 
+import Control.Concurrent
+import Control.Concurrent.STM
+import qualified Control.Exception as E
 import qualified Data.ByteString as BS
 import qualified Data.ByteString.Char8 as C8
 import qualified Data.ByteString.Short as Short
+import qualified Data.ByteString.UTF8 as UTF8
 import Data.IORef
 import Network.Control
-import UnliftIO.Concurrent
-import qualified UnliftIO.Exception as E
-import UnliftIO.STM
+import Network.HTTP.Semantics
 
 import Imports hiding (delete, insert)
-import Network.HPACK
-import Network.HPACK.Token
 import Network.HTTP2.Frame
 import Network.HTTP2.H2.Context
 import Network.HTTP2.H2.EncodeFrame
@@ -38,60 +41,20 @@ continuationLimit = 10
 headerFragmentLimit :: Int
 headerFragmentLimit = 51200 -- 50K
 
-pingRateLimit :: Int
-pingRateLimit = 4
-
-settingsRateLimit :: Int
-settingsRateLimit = 4
-
-emptyFrameRateLimit :: Int
-emptyFrameRateLimit = 4
-
-rstRateLimit :: Int
-rstRateLimit = 4
-
 ----------------------------------------------------------------
 
 frameReceiver :: Context -> Config -> IO ()
-frameReceiver ctx@Context{..} conf@Config{..} = loop 0 `E.catch` sendGoaway
+frameReceiver ctx conf@Config{..} = do
+    labelMe "H2 receiver"
+    loop
   where
-    loop :: Int -> IO ()
-    loop n
-        | n == 6 = do
-            yield
-            loop 0
-        | otherwise = do
-            hd <- confReadN frameHeaderLength
-            if BS.null hd
-                then enqueueControl controlQ $ CFinish ConnectionIsClosed
-                else do
-                    processFrame ctx conf $ decodeFrameHeader hd
-                    loop (n + 1)
-
-    sendGoaway se
-        | Just e@ConnectionIsClosed <- E.fromException se =
-            enqueueControl controlQ $ CFinish e
-        | Just e@(ConnectionErrorIsReceived _ _ _) <- E.fromException se =
-            enqueueControl controlQ $ CFinish e
-        | Just e@(ConnectionErrorIsSent err sid msg) <- E.fromException se = do
-            let frame = goawayFrame sid err $ Short.fromShort msg
-            enqueueControl controlQ $ CFrames Nothing [frame]
-            enqueueControl controlQ $ CFinish e
-        | Just e@(StreamErrorIsSent err sid msg) <- E.fromException se = do
-            let frame = resetFrame err sid
-            enqueueControl controlQ $ CFrames Nothing [frame]
-            let frame' = goawayFrame sid err $ Short.fromShort msg
-            enqueueControl controlQ $ CFrames Nothing [frame']
-            enqueueControl controlQ $ CFinish e
-        | Just e@(StreamErrorIsReceived err sid) <- E.fromException se = do
-            let frame = goawayFrame sid err "treat a stream error as a connection error"
-            enqueueControl controlQ $ CFrames Nothing [frame]
-            enqueueControl controlQ $ CFinish e
-        -- this never happens
-        | Just e@(BadThingHappen _) <- E.fromException se =
-            enqueueControl controlQ $ CFinish e
-        | otherwise =
-            enqueueControl controlQ $ CFinish $ BadThingHappen se
+    loop = do
+        -- If 'confReadN' is timeouted, an exception is thrown
+        -- to destroy the thread trees.
+        hd <- confReadN frameHeaderLength
+        when (BS.null hd) $ E.throwIO ConnectionIsTimeout
+        processFrame ctx conf $ decodeFrameHeader hd
+        loop
 
 ----------------------------------------------------------------
 
@@ -175,7 +138,7 @@ controlOrStream ctx@Context{..} Config{..} ftyp header@FrameHeader{streamId, pay
 processState :: StreamState -> Context -> Stream -> StreamId -> IO Bool
 -- Transition (process1)
 processState (Open _ (NoBody tbl@(_, reqvt))) ctx@Context{..} strm@Stream{streamInput} streamId = do
-    let mcl = fst <$> (getHeaderValue tokenContentLength reqvt >>= C8.readInt)
+    let mcl = fst <$> (getFieldValue tokenContentLength reqvt >>= C8.readInt)
     when (just mcl (/= (0 :: Int))) $
         E.throwIO $
             StreamErrorIsSent
@@ -183,21 +146,22 @@ processState (Open _ (NoBody tbl@(_, reqvt))) ctx@Context{..} strm@Stream{stream
                 streamId
                 "no body but content-length is not zero"
     tlr <- newIORef Nothing
-    let inpObj = InpObj tbl (Just 0) (return "") tlr
+    let inpObj = InpObj tbl (Just 0) (return (mempty, True)) tlr
     if isServer ctx
         then do
-            let si = toServerInfo roleInfo
-            atomically $ writeTQueue (inputQ si) $ Input strm inpObj
+            let ServerInfo{..} = toServerInfo roleInfo
+            launch ctx strm inpObj
         else putMVar streamInput $ Right inpObj
     halfClosedRemote ctx strm
     return False
 
 -- Transition (process2)
-processState (Open hcl (HasBody tbl@(_, reqvt))) ctx@Context{..} strm@Stream{streamInput} _streamId = do
-    let mcl = fst <$> (getHeaderValue tokenContentLength reqvt >>= C8.readInt)
+processState (Open hcl (HasBody tbl@(_, reqvt))) ctx@Context{..} strm@Stream{streamInput, streamRxQ} _streamId = do
+    let mcl = fst <$> (getFieldValue tokenContentLength reqvt >>= C8.readInt)
     bodyLength <- newIORef 0
     tlr <- newIORef Nothing
     q <- newTQueueIO
+    writeIORef streamRxQ $ Just q
     setStreamState ctx strm $ Open hcl (Body q mcl bodyLength tlr)
     -- FLOW CONTROL: WINDOW_UPDATE 0: recv: announcing my limit properly
     -- FLOW CONTROL: WINDOW_UPDATE: recv: announcing my limit properly
@@ -205,8 +169,8 @@ processState (Open hcl (HasBody tbl@(_, reqvt))) ctx@Context{..} strm@Stream{str
     let inpObj = InpObj tbl mcl (readSource bodySource) tlr
     if isServer ctx
         then do
-            let si = toServerInfo roleInfo
-            atomically $ writeTQueue (inputQ si) $ Input strm inpObj
+            let ServerInfo{..} = toServerInfo roleInfo
+            launch ctx strm inpObj
         else putMVar streamInput $ Right inpObj
     return False
 
@@ -283,13 +247,17 @@ getOddStream ctx ftyp streamId Nothing
                     let errmsg =
                             Short.toShort
                                 ( "this frame is not allowed in an idle stream: "
-                                    `BS.append` (C8.pack (show ftyp))
+                                    `BS.append` C8.pack (show ftyp)
                                 )
                     E.throwIO $ ConnectionErrorIsSent ProtocolError streamId errmsg
                 when (ftyp == FrameHeaders) $ setPeerStreamID ctx streamId
                 -- FLOW CONTROL: SETTINGS_MAX_CONCURRENT_STREAMS: recv: rejecting if over my limit
                 Just <$> openOddStreamCheck ctx streamId ftyp
-    | otherwise = undefined -- never reach
+    | otherwise =
+        -- We received a frame from the server on an unknown stream
+        -- (likely a previously created and then subsequently reset stream).
+        -- We just drop it.
+        return Nothing
 
 ----------------------------------------------------------------
 
@@ -307,7 +275,7 @@ control FrameSettings header@FrameHeader{flags, streamId} bs Context{myFirstSett
         else do
             -- Settings Flood - CVE-2019-9515
             rate <- getRate settingsRate
-            when (rate > settingsRateLimit) $
+            when (rate > settingsRateLimit mySettings) $
                 E.throwIO $
                     ConnectionErrorIsSent EnhanceYourCalm streamId "too many settings"
             let ack = settingsFrame setAck []
@@ -318,16 +286,15 @@ control FrameSettings header@FrameHeader{flags, streamId} bs Context{myFirstSett
                     enqueueControl controlQ setframe
                 else do
                     -- Server side only
-                    connRxWS <- rxfWindow <$> readIORef rxFlow
+                    connRxWS <- rxfBufSize <$> readIORef rxFlow
                     let frames = makeNegotiationFrames mySettings connRxWS
                         setframe = CFrames (Just peerAlist) (frames ++ [ack])
                     writeIORef myFirstSettings True
                     enqueueControl controlQ setframe
-control FramePing FrameHeader{flags, streamId} bs Context{controlQ, pingRate} =
+control FramePing FrameHeader{flags, streamId} bs Context{mySettings, controlQ, pingRate} =
     unless (testAck flags) $ do
-        -- Ping Flood - CVE-2019-9512
         rate <- getRate pingRate
-        if rate > pingRateLimit
+        if rate > pingRateLimit mySettings
             then E.throwIO $ ConnectionErrorIsSent EnhanceYourCalm streamId "too many ping"
             else do
                 let frame = pingFrame bs
@@ -365,12 +332,12 @@ push header@FrameHeader{streamId} bs ctx = do
     (_, vt) <- hpackDecodeHeader frag streamId ctx
     let ClientInfo{..} = toClientInfo $ roleInfo ctx
     when
-        ( getHeaderValue tokenAuthority vt == Just authority
-            && getHeaderValue tokenScheme vt == Just scheme
+        ( getFieldValue tokenAuthority vt == Just (UTF8.fromString authority)
+            && getFieldValue tokenScheme vt == Just scheme
         )
         $ do
-            let mmethod = getHeaderValue tokenMethod vt
-                mpath = getHeaderValue tokenPath vt
+            let mmethod = getFieldValue tokenMethod vt
+                mpath = getFieldValue tokenPath vt
             case (mmethod, mpath) of
                 (Just method, Just path) ->
                     -- FLOW CONTROL: SETTINGS_MAX_CONCURRENT_STREAMS: recv: rejecting if over my limit
@@ -411,7 +378,7 @@ stream FrameHeaders header@FrameHeader{flags, streamId} bs ctx s@(Open hcl JustO
         then do
             -- Empty Frame Flooding - CVE-2019-9518
             rate <- getRate $ emptyFrameRate ctx
-            if rate > emptyFrameRateLimit
+            if rate > emptyFrameRateLimit (mySettings ctx)
                 then
                     E.throwIO $
                         ConnectionErrorIsSent EnhanceYourCalm streamId "too many empty headers"
@@ -441,10 +408,9 @@ stream FrameHeaders header@FrameHeader{flags, streamId} bs ctx (Open _ (Body q _
         then do
             tbl <- hpackDecodeTrailer frag streamId ctx
             writeIORef tlr (Just tbl)
-            atomically $ writeTQueue q $ Right ""
+            atomically $ writeTQueue q $ Right (mempty, True)
             return HalfClosedRemote
         else -- we don't support continuation here.
-
             E.throwIO $
                 ConnectionErrorIsSent
                     ProtocolError
@@ -456,7 +422,7 @@ stream
     FrameData
     header@FrameHeader{flags, payloadLength, streamId}
     bs
-    Context{emptyFrameRate, rxFlow}
+    Context{emptyFrameRate, rxFlow, mySettings}
     s@(Open _ (Body q mcl bodyLength _))
     Stream{..} = do
         DataFrame body <- guardIt $ decodeDataFrame header bs
@@ -483,11 +449,11 @@ stream
         if body == ""
             then unless endOfStream $ do
                 rate <- getRate emptyFrameRate
-                when (rate > emptyFrameRateLimit) $ do
+                when (rate > emptyFrameRateLimit mySettings) $ do
                     E.throwIO $ ConnectionErrorIsSent EnhanceYourCalm streamId "too many empty data"
             else do
                 writeIORef bodyLength len
-                atomically $ writeTQueue q $ Right body
+                atomically $ writeTQueue q $ Right (body, endOfStream)
         if endOfStream
             then do
                 case mcl of
@@ -500,7 +466,7 @@ stream
                                     streamId
                                     "actual body length is not the same as content-length"
                 -- no trailers
-                atomically $ writeTQueue q $ Right ""
+                atomically $ writeTQueue q $ Right (mempty, True)
                 return HalfClosedRemote
             else return s
 
@@ -511,7 +477,7 @@ stream FrameContinuation FrameHeader{flags, streamId} frag ctx s@(Open hcl (Cont
         then do
             -- Empty Frame Flooding - CVE-2019-9518
             rate <- getRate $ emptyFrameRate ctx
-            if rate > emptyFrameRateLimit
+            if rate > emptyFrameRateLimit (mySettings ctx)
                 then
                     E.throwIO $
                         ConnectionErrorIsSent EnhanceYourCalm streamId "too many empty continuation"
@@ -547,28 +513,48 @@ stream FrameWindowUpdate header bs _ s strm = do
 stream FrameRSTStream header@FrameHeader{streamId} bs ctx s strm = do
     -- Rapid Rest: CVE-2023-44487
     rate <- getRate $ rstRate ctx
-    when (rate > rstRateLimit) $
+    when (rate > rstRateLimit (mySettings ctx)) $
         E.throwIO $
             ConnectionErrorIsSent EnhanceYourCalm streamId "too many rst_stream"
     RSTStreamFrame err <- guardIt $ decodeRSTStreamFrame header bs
     let cc = Reset err
+    closed ctx strm cc
 
-    -- The spec mandates (section 8.1):
+    -- HTTP2 spec, section 5.1, "Stream States":
     --
-    -- > When this is true, a server MAY request that the client abort
-    -- > transmission of a request without error by sending a RST_STREAM with an
-    -- > error code of NO_ERROR after sending a complete response (i.e., a frame
-    -- > with the END_STREAM flag).
+    -- > A stream in the "open" state may be used by both peers to send frames
+    -- > of any type. (..) From this state, either endpoint can send a frame
+    -- > with an END_STREAM flag set, which causes the stream to transition into
+    -- > one of the "half-closed" states.  An endpoint sending an END_STREAM
+    -- > flag causes the stream state to become "half-closed (local)"; an
+    -- > endpoint receiving an END_STREAM flag causes the stream state to become
+    -- > "half-closed (remote)".
     --
-    -- We check the first part ("after sending a complete response") by checking
-    -- the current stream state.
-    case (s, err) of
-        (HalfClosedRemote, NoError) ->
-            return (Closed cc)
+    -- Crucially (for the specific case we're dealing with here), it continues:
+    --
+    -- > /Either endpoint/ can send a RST_STREAM frame from this state, causing
+    -- > it to transition immediately to "closed".
+    --
+    -- (emphasis not in original).
+    --
+    -- In addition, the spec states (about the open state):
+    --
+    -- > Either endpoint can send a RST_STREAM frame from this state, causing it
+    -- > to transition immediately to "closed".
+    --
+    -- This justifies the two non-error cases, below. (Section 8.1 of the spec
+    -- is also relevant, but it is less explicit about the /either endpoint/
+    -- part.)
+    case s of
+        Open _ _
+            | isNonCritical err ->
+                -- Open /or/ half-closed (local)
+                return (Closed cc)
+        HalfClosedRemote
+            | isNonCritical err ->
+                return (Closed cc)
         _otherwise -> do
-            closed ctx strm cc
             E.throwIO $ StreamErrorIsReceived err streamId
-
 -- (No state transition)
 stream FramePriority header bs _ s Stream{streamNumber} = do
     -- ignore
@@ -598,44 +584,74 @@ stream x FrameHeader{streamId} _ _ _ _ =
         StreamErrorIsSent ProtocolError streamId $
             fromString ("illegal frame " ++ show x ++ " for " ++ show streamId)
 
+{- FOURMOLU_DISABLE -}
+-- Although some stream errors indicate misbehaving peers, such as
+-- FLOW_CONTROL_ERROR, not all errors do. We will close the connection only
+-- for critical errors.
+isNonCritical :: ErrorCode -> Bool
+isNonCritical NoError       = True
+isNonCritical Cancel        = True
+isNonCritical InternalError = True
+isNonCritical _             = False
+{- FOURMOLU_ENABLE -}
+
 ----------------------------------------------------------------
 
 -- | Type for input streaming.
-data Source
-    = Source
-        (Int -> IO ())
-        (TQueue (Either E.SomeException ByteString))
-        (IORef ByteString)
-        (IORef Bool)
+data Source = Source RxQ (Int -> IO ()) (IORef Bool)
 
-mkSource
-    :: TQueue (Either E.SomeException ByteString) -> (Int -> IO ()) -> IO Source
-mkSource q inform = Source inform q <$> newIORef "" <*> newIORef False
+mkSource :: RxQ -> (Int -> IO ()) -> IO Source
+mkSource q inform = Source q inform <$> newIORef False
 
-readSource :: Source -> IO ByteString
-readSource (Source inform q refBS refEOF) = do
+readSource :: Source -> IO (ByteString, Bool)
+readSource (Source q inform refEOF) = do
     eof <- readIORef refEOF
     if eof
-        then return ""
+        then return (mempty, True)
         else do
-            bs <- readBS
-            let len = BS.length bs
-            inform len
-            return bs
+            mBS <- atomically $ readTQueue q
+            case mBS of
+                Left err -> do
+                    writeIORef refEOF True
+                    E.throwIO err
+                Right (bs, isEOF) -> do
+                    writeIORef refEOF isEOF
+                    let len = BS.length bs
+                    inform len
+                    return (bs, isEOF)
+
+----------------------------------------------------------------
+
+closureClient :: Config -> Either E.SomeException a -> IO a
+closureClient Config{..} (Right x) = do
+    let frame = goawayFrame 0 NoError ""
+    confSendAll frame `E.catch` ignore
+    return x
   where
-    readBS :: IO ByteString
-    readBS = do
-        bs0 <- readIORef refBS
-        if bs0 == ""
-            then do
-                mBS <- atomically $ readTQueue q
-                case mBS of
-                    Left err -> do
-                        writeIORef refEOF True
-                        E.throwIO err
-                    Right bs -> do
-                        when (bs == "") $ writeIORef refEOF True
-                        return bs
-            else do
-                writeIORef refBS ""
-                return bs0
+    ignore (E.SomeException e)
+        | isAsyncException e = E.throwIO e
+        | otherwise = return ()
+closureClient conf (Left se) = closureServer conf se
+
+closureServer :: Config -> E.SomeException -> IO a
+closureServer Config{..} se
+    | isAsyncException se = E.throwIO se
+    | Just ConnectionIsClosed <- E.fromException se = do
+        E.throwIO ConnectionIsClosed
+    | Just e@(ConnectionErrorIsReceived{}) <- E.fromException se =
+        E.throwIO e
+    | Just e@(ConnectionErrorIsSent err sid msg) <- E.fromException se = do
+        let frame = goawayFrame sid err $ Short.fromShort msg
+        confSendAll frame
+        E.throwIO e
+    | Just e@(StreamErrorIsSent err sid msg) <- E.fromException se = do
+        let frame = resetFrame err sid
+        let frame' = goawayFrame sid err $ Short.fromShort msg
+        confSendAll $ frame <> frame'
+        E.throwIO e
+    | Just e@(StreamErrorIsReceived err sid) <- E.fromException se = do
+        let frame = goawayFrame sid err "treat a stream error as a connection error"
+        confSendAll frame
+        E.throwIO e
+    | Just (_ :: HTTP2Error) <- E.fromException se = E.throwIO se
+    | otherwise = E.throwIO $ BadThingHappen se
