@@ -9,6 +9,8 @@ module Network.HTTP2.Server.Internal (
     runIO,
 ) where
 
+import Network.HTTP.Semantics.Server
+import Network.HTTP.Semantics.Server.Internal
+
 import Network.HTTP2.H2
 import Network.HTTP2.Server.Run
-import Network.HTTP2.Server.Types
