@@ -11,7 +11,7 @@ import HPACK.HeaderBlock
 spec :: Spec
 spec = do
     describe "fromHeaderBlock" $ do
-        it "decodes HeaderList in request" $ do
+        it "decodes [Header] in request" $ do
             withDynamicTableForDecoding 4096 4096 $ \dyntabl -> do
                 h1 <- decodeHeader dyntabl d41b
                 h1 `shouldBe` d41h
@@ -19,7 +19,7 @@ spec = do
                 h2 `shouldBe` d42h
                 h3 <- decodeHeader dyntabl d43b
                 h3 `shouldBe` d43h
-        it "decodes HeaderList in response" $ do
+        it "decodes [Header] in response" $ do
             withDynamicTableForDecoding 256 4096 $ \dyntabl -> do
                 h1 <- decodeHeader dyntabl d61b
                 h1 `shouldBe` d61h
@@ -27,11 +27,11 @@ spec = do
                 h2 `shouldBe` d62h
                 h3 <- decodeHeader dyntabl d63b
                 h3 `shouldBe` d63h
-        it "decodes HeaderList in response (deny max table size update to 0)" $
+        it "decodes [Header] in response (deny max table size update to 0)" $
             withDynamicTableForDecoding 256 4096 $ \dyntabl -> do
                 h1 <- decodeHeader dyntabl d81b
                 h1 `shouldBe` d81h
-        it "decodes HeaderList even if an entry is larger than DynamicTable" $
+        it "decodes [Header] even if an entry is larger than DynamicTable" $
             withDynamicTableForEncoding 64 $ \etbl ->
                 withDynamicTableForDecoding 64 4096 $ \dtbl -> do
                     hs <- encodeHeader defaultEncodeStrategy 4096 etbl hl1
@@ -40,7 +40,7 @@ spec = do
                     isDynamicTableEmpty etbl `shouldReturn` True
                     isDynamicTableEmpty dtbl `shouldReturn` True
 
-hl1 :: HeaderList
+hl1 :: [Header]
 hl1 =
     [ ("custom-key", "custom-value")
     ,
