@@ -1,21 +1,19 @@
-{-# LANGUAGE BangPatterns #-}
 {-# LANGUAGE LambdaCase #-}
 {-# LANGUAGE OverloadedStrings #-}
 {-# LANGUAGE RankNTypes #-}
 
-module HTTP2.ClientSpec where
+module HTTP2.ClientSpec (spec) where
 
 import Control.Concurrent
 import qualified Control.Exception as E
 import Control.Monad
-import Data.ByteString (ByteString)
 import Data.ByteString.Builder (byteString)
-import qualified Data.ByteString.Char8 as C8
 import Data.Foldable (for_)
 import Data.Maybe
 import Data.Traversable (for)
+import Network.HTTP.Semantics
 import Network.HTTP.Types
-import Network.Run.TCP
+import Network.Run.TCP hiding (defaultSettings)
 import System.IO.Unsafe (unsafePerformIO)
 import System.Random
 import System.Timeout (timeout)
@@ -32,27 +30,24 @@ port = show $ unsafePerformIO (randomPort <$> getStdGen)
 host :: String
 host = "127.0.0.1"
 
-host' :: ByteString
-host' = C8.pack host
-
 spec :: Spec
 spec = do
     describe "client" $ do
         it "receives an error if scheme is missing" $
             E.bracket (forkIO $ runServer defaultServer) killThread $ \_ -> do
                 threadDelay 10000
-                runClient "" host' (defaultClient []) `shouldThrow` connectionError
+                runClient "" host (defaultClient []) `shouldThrow` streamError
 
         it "receives an error if authority is missing" $
             E.bracket (forkIO $ runServer defaultServer) killThread $ \_ -> do
                 threadDelay 10000
-                runClient "http" "" (defaultClient []) `shouldThrow` connectionError
+                runClient "http" "" (defaultClient []) `shouldThrow` streamError
 
         it "receives an error if authority and host are different" $
             E.bracket (forkIO $ runServer defaultServer) killThread $ \_ -> do
                 threadDelay 10000
-                runClient "http" host' (defaultClient [("Host", "foo")])
-                    `shouldThrow` connectionError
+                runClient "http" host (defaultClient [("Host", "foo")])
+                    `shouldThrow` streamError
 
         it "does not deadlock (in concurrent setting)" $
             E.bracket (forkIO $ runServer irresponsiveServer) killThread $ \_ -> do
@@ -72,7 +67,7 @@ spec = do
                 let maxConc = fromJust $ maxConcurrentStreams defaultSettings
 
                 resultVars <- runClient "http" "localhost" $ \sendReq aux -> do
-                    for [1 .. (maxConc + 1) :: Int] $ \_ -> do
+                    replicateM ((maxConc + 1) :: Int) $ do
                         resultVar <- newEmptyMVar
                         concurrentClient resultVar sendReq aux
                         pure resultVar
@@ -115,7 +110,7 @@ responseHello = S.responseBuilder ok200 header body
     body = byteString "Hello, world!\n"
 
 runClient :: Scheme -> Authority -> Client a -> IO a
-runClient sc au client = runTCPClient host port $ runHTTP2Client
+runClient sc au client = runTCPClient host port runHTTP2Client
   where
     cliconf = defaultClientConfig{scheme = sc, authority = au}
     runHTTP2Client s =
@@ -141,6 +136,7 @@ concurrentClient resultVar sendRequest _aux = do
         putMVar resultVar result
     threadDelay 10000
 
-connectionError :: Selector HTTP2Error
-connectionError ConnectionErrorIsReceived{} = True
-connectionError _ = False
+streamError :: Selector HTTP2Error
+streamError StreamErrorIsReceived{} = True
+streamError ConnectionErrorIsReceived{} = True
+streamError _ = False
